import {Stack, Typography, Box, Grid2 as Grid, Button, MenuItem, Select, FormControl, InputLabel} from '@mui/material'
import {useShow} from '@refinedev/core'
import {Show, TextFieldComponent as TextField} from '@refinedev/mui'

import {useEffect, useState} from 'react'
import axios from 'axios'

import {BACKEND_URL} from '../../lib/constants'

type SightItem = {
  id: number
  name: string
  latitude: number
  longitude: number
  city_id: number
  city: string
  [key: string]: string | number
}

export const StationShow = () => {
  const {query} = useShow({})
  const {data, isLoading} = query
  const record = data?.data

  const [sights, setSights] = useState<SightItem[]>([])
  const [linkedSights, setLinkedSights] = useState<SightItem[]>([])
  const [selectedSightId, setSelectedSightId] = useState<number | ''>('')
  const [sightsLoading, setSightsLoading] = useState<boolean>(true)

  useEffect(() => {
    if (record?.id) {
      axios
        .get(`${BACKEND_URL}/station/${record.id}/sight`)
        .then((response) => {
          setLinkedSights(response?.data || [])
        })
        .catch(() => {
          setLinkedSights([])
        })
    }
  }, [record?.id])

  useEffect(() => {
    axios
      .get(`${BACKEND_URL}/sight/`) // without "/" throws CORS error
      .then((response) => {
        setSights(response?.data || [])
        setSightsLoading(false)
      })
      .catch(() => {
        setSights([])
        setSightsLoading(false)
      })
  }, [])

  const availableSights = sights.filter((sight) => !linkedSights.some((linked) => linked.id === sight.id))

  const linkSight = () => {
    if (selectedSightId) {
      axios
        .post(
          `${BACKEND_URL}/station/${record?.id}/sight`,
          {sight_id: selectedSightId},
          {
            headers: {
              accept: 'application/json',
              'Content-Type': 'application/json',
            },
          },
        )
        .then(() => {
          axios
            .get(`${BACKEND_URL}/station/${record?.id}/sight`)
            .then((response) => {
              setLinkedSights(response?.data || [])
            })
            .catch(() => {
              setLinkedSights([])
            })
        })
        .catch((error) => {
          console.error('Error linking sight:', error)
        })
    }
  }

  const deleteSight = (sightId: number) => {
    axios
      .delete(`${BACKEND_URL}/station/${record?.id}/sight`, {
        data: {sight_id: sightId},
      })
      .then(() => {
        setLinkedSights((prevSights) => prevSights.filter((item) => item.id !== sightId))
      })
      .catch((error) => {
        console.error('Error deleting sight:', error)
      })
  }

  const fields = [
    // {label: 'ID', data: 'id'},
    {label: 'Название', data: 'name'},
    // {label: 'Широта', data: 'latitude'}, #
    // {label: 'Долгота', data: 'longitude'}, #
    {label: 'Описание', data: 'description'},
  ]

  const sightFields: Array<{label: string; data: keyof SightItem}> = [
    // {label: 'ID', data: 'id'},
    {label: 'Название', data: 'name'},
    // {label: 'Широта', data: 'latitude'}, #
    // {label: 'Долгота', data: 'longitude'}, #
    // {label: 'ID города', data: 'city_id'},
    {label: 'Город', data: 'city'},
  ]

  return (
    <Show isLoading={isLoading}>
      <Stack gap={4}>
        {fields.map(({label, data}) => (
          <Stack key={data} gap={1}>
            <Typography variant="body1" fontWeight="bold">
              {label}
            </Typography>
            <TextField value={record?.[data] || ''} />
          </Stack>
        ))}

        <Stack gap={2}>
          <Typography variant="body1" fontWeight="bold">
            Привязанные достопримечательности
          </Typography>

          <Grid container gap={2}>
            {sightsLoading ? (
              <Typography>Загрузка достопримечательностей...</Typography>
            ) : linkedSights.length > 0 ? (
              linkedSights.map((sight, index) => (
                <Box
                  key={index}
                  sx={{
                    marginBottom: '8px',
                    padding: '14px',
                    borderRadius: 2,
                    border: (theme) => `2px solid ${theme.palette.divider}`,
                  }}
                >
                  <Stack gap={0.5}>
                    {sightFields.map(({label, data}) => (
                      <Typography key={data}>
                        <strong>{label}:</strong> {sight[data]}
                      </Typography>
                    ))}

                    <Button variant="outlined" color="error" onClick={() => deleteSight(sight?.id)} sx={{mt: 1.5}}>
                      Отвязать
                    </Button>
                  </Stack>
                </Box>
              ))
            ) : (
              <Typography>Достопримечательности не найдены</Typography>
            )}
          </Grid>

          <Stack gap={2}>
            <Typography variant="body1" fontWeight="bold">
              Привязать достопримечательность
            </Typography>

            <FormControl fullWidth>
              <InputLabel>Достопримечательность</InputLabel>
              <Select value={selectedSightId} onChange={(e) => setSelectedSightId(Number(e.target.value))} label="Достопримечательность" fullWidth>
                {availableSights.map((sight) => (
                  <MenuItem key={sight.id} value={sight.id}>
                    {sight.name}
                  </MenuItem>
                ))}
              </Select>
            </FormControl>

            <Button variant="contained" onClick={linkSight} disabled={!selectedSightId}>
              Привязать
            </Button>
          </Stack>
        </Stack>
      </Stack>
    </Show>
  )
}
